@@ -3,7 +3,7 @@ import logging
 import json
 from inspect import getmembers, isclass
 import asyncio
-
+from enum import Enum
 from pydantic import BaseSettings
 
 from elasticsearch import Elasticsearch
@@ -64,21 +64,19 @@ def get_model_enum_values():
     return model_enums
 
 
-"""
-def get_model_enum_values2():
-    model_enums = {}
-    for data_type_name in dir(model_module):
-        if not data_type_name.startswith('__') and data_type_name != 'Enum' and type(model_module.__dict__[data_type_name]) == enum.EnumMeta:
-            model_enums[data_type_name] = [
-                {
-                    '_key': variable_name,
-                    'en': model_en_module.__dict__[data_type_name].__dict__[variable_name].value,
-                    'zh': model_zh_module.__dict__[data_type_name].__dict__[variable_name].value,
-                }
-                for variable_name in dir(model_module.__dict__[data_type_name]) if not variable_name.startswith('__')
-            ]
-    return model_enums
-"""
+#def get_model_enum_values2():
+#    model_enums = {}
+#    for data_type_name in dir(model_module):
+#        if not data_type_name.startswith('__') and data_type_name != 'Enum' and type(model_module.__dict__[data_type_name]) == enum.EnumMeta:
+#            model_enums[data_type_name] = [
+#                {
+#                    '_key': variable_name,
+#                    'en': model_en_module.__dict__[data_type_name].__dict__[variable_name].value,
+#                    'zh': model_zh_module.__dict__[data_type_name].__dict__[variable_name].value,
+#                }
+#                for variable_name in dir(model_module.__dict__[data_type_name]) if not variable_name.startswith('__')
+#            ]
+#    return model_enums
 
 
 class Settings(BaseSettings):
@@ -105,7 +103,7 @@ token_verifier = Auth0TokenVerifier()
 token_cache_redis = None  # TODO rework the global clients.
 
 
-def _get_uid(token: str):
+def _get_token_payload(token: str) -> str:
     global token_cache_redis
     if settings.redis_host:
         if token_cache_redis is None:
@@ -113,26 +111,35 @@ def _get_uid(token: str):
                 settings.redis_host, settings.redis_port,
                 db=settings.token_cache_db_id,
                 password=settings.redis_password
-                )
+            )
         cached_value = token_cache_redis.get(token)
         if cached_value is not None:
-            return json.loads(cached_value.decode('utf-8')).get(f"{settings.auth0_namespace}uid")
+            return cached_value.decode('utf-8')
     try:
         payload = token_verifier.verify(token)
         if payload.get("status") == 'error':
+            log.warn('Payload status == error payload: %s, token: %s', payload, token, exc_info=True)
             raise IllegalTokenExcpetion()
-        uid = payload.get(f"{settings.auth0_namespace}uid")
-        log.warn('uid retrieved is %s', uid)
-        if token_cache_redis is not None and uid:
+        payload_str = json.dumps(payload, default=str)
+        if token_cache_redis is not None and payload:
             ttl = 86400 * 1000  # hard code 1 day, ttl value is in milliseconds
             token_cache_redis.setex(
-                token, ttl, json.dumps(payload, default=str))
-        return uid
+                token, ttl, payload_str)
+        return payload_str
     except Exception as e:
         log.warn('Token verification failed', exc_info=True)
         raise IllegalTokenExcpetion(e)
 
 
+def _get_uid(token: str):
+    payload_str = _get_token_payload(token)
+    return json.loads(payload_str).get(f"{settings.auth0_namespace}uid")
+
+
+def _get_sub(token: str):
+    payload_str = _get_token_payload(token)
+    return json.loads(payload_str).get(f"sub")
+
 async def validate_socket_token(token: str, websocket: WebSocket):
     global token_cache_redis
     if settings.redis_host:
@@ -141,7 +148,7 @@ async def validate_socket_token(token: str, websocket: WebSocket):
                 settings.redis_host, settings.redis_port,
                 db=settings.token_cache_db_id,
                 password=settings.redis_password
-                )
+            )
         cached_value = token_cache_redis.get(token)
         if cached_value is not None:
             payload = json.loads(cached_value.decode('utf-8'))
@@ -173,7 +180,7 @@ def get_uid(token: str, token_verifier: object):
                 settings.redis_host, settings.redis_port,
                 db=settings.token_cache_db_id,
                 password=settings.redis_password
-                )
+            )
         cached_value = token_cache_redis.get(token.credentials)
         if cached_value is not None:
             return json.loads(cached_value.decode('utf-8')).get(f"{settings.auth0_namespace}uid")
@@ -186,7 +193,7 @@ def get_uid(token: str, token_verifier: object):
         log.warn('domain %s', domain)
         users = Auth0Users(domain)
         userinfo = users.userinfo(token.credentials)
-        log.warn('userinfo %s', userinfo);
+        log.warn('userinfo %s', userinfo)
         uid = userinfo.get(f"{settings.auth0_namespace}uid")
         # value = result.get(f"{settings.auth0_namespace}uid")
         # log.warn('uid retrieved is %s', value)
@@ -229,8 +236,9 @@ async def redis_connector(
 ):
     async def consumer_handler(ws: WebSocket, redis_client: object):
         try:
-            own_channel = (await redis_client.subscribe(wallet))[0]
-            asyncio.create_task(ws_reader(own_channel, ws))
+            if wallet:
+                own_channel = (await redis_client.subscribe(wallet))[0]
+                asyncio.create_task(ws_reader(own_channel, ws))
             additional_channel = None
             while True:
                 message = await ws.receive_text()
@@ -262,6 +270,71 @@ async def redis_connector(
     redis.close()
     await redis.wait_closed()
 
+async def calculate_referral_incomes(es, account_id: str):
+    """
+    return aggregated referral incomes data by currency
+    e.g.
+    {
+        "UDST" : 40,
+        "BDC": 20
+    }
+    return {} if no referral txn match the account id is found
+    """
+    log.warning(f"Calculating referral incomes for account id: {account_id}")
+    query = {
+        '_source':['referral_amount', 'referral_currency', 'booking.provider_uid', 'booking.consumer_uid', 'timestamp'], # Return these fields only
+        "query": {
+            "bool": {
+                "must": [
+                    {
+                        "match": {
+                            "type": "referral_txn_history"
+                        }
+                    },
+                    {
+                        "match": {
+                            "referral_uid": account_id
+                        }
+                    }
+                ]
+            }
+        }
+    }
+    resp = await es.search(body=query)
+    if resp == None or resp['size'] == 0:
+        return {}
+    
+    aggs = {}
+    for i in resp['results']:
+        if i['referral_currency'] not in aggs.keys():
+            aggs[i['referral_currency']] = i['referral_amount']
+        else:
+            aggs[i['referral_currency']] += i['referral_amount']
+
+    return aggs
+
+async def list_referral_incomes(es, account_id: str):
+    log.warning(f"listing referral incomes for account id: {account_id}")
+    query = {
+        "query": {
+            "bool": {
+                "must": [
+                    {
+                        "match": {
+                            "type": "referral_txn_history"
+                        }
+                    },
+                    {
+                        "match": {
+                            "referral_uid": account_id
+                        }
+                    }
+                ]
+            }
+        }
+    }
+    resp = await es.search(body=query)
+    return resp['results']
 
 def create_app():
     app = FastAPI(title=settings.app_name)
@@ -321,12 +394,44 @@ def create_app():
             allow_credentials=True,
             allow_methods=["*"],
             allow_headers=["*"],
+            max_age=7200
         )
     return app
 
 
+class PublishAction(Enum):
+    message = 'message'
+    data = 'data'
+    action = 'action'
+
+
 def publish_message(message: Union[Dict[Any, Any], str], wallet: str, sync_redis: object = None):
-    """Publish a message to a user identified by wallet"""
+    """Publish a message to a user identified by wallet
+    mesage format: 
+    {
+        "channel": "xxxx", # what channel this message was broadcasted to. Only clients subscribed to this channel can receive this message. Yet as one client may subscribe to multiple channels, this field is kept to notify which channel this is about.
+        "type:": "message|data|action", 
+        # when type is data, means the user (uid is defined by channel) had related data change.
+        # when type is message, means the system want some infromation displayed in real time on the user's app if the user is online. Body is fixed format { "message": message }
+        # when type is action, ...we havent' decided what to do.
+        "body": {
+        } # the message body. 
+            # for message type, body is like { "message": message }
+            # for data type, body is like {"slot_update": ["slot_ids"]} / {"slot_locked": ["slot_ids"]} /{"booking_fulfilled": ["booking_id"]} etc.
+            # for action type, body is not defined yet.
+        }
+
+    The caller has to put message like 
+    publish_message(dict(
+        action=PublishAction.data.name,
+        body={...}
+        ), wallet=user_id)
+
+    or if simply just push a text message
+    publish_message("Please display this to the user", wallet=user_id)
+
+    """
+    log.info("--- api_common.py:publish_message()")
     if not sync_redis:
         sync_redis = redis.Redis(
             host=settings.redis_host,
@@ -336,7 +441,12 @@ def publish_message(message: Union[Dict[Any, Any], str], wallet: str, sync_redis
         )
     if isinstance(message, Dict):
         sync_redis.publish(wallet, json.dumps(message, default=str))
+        log.info(
+            f"--- api_common.py:publish_message(): wallet={wallet}, message={json.dumps(message, default=str)}")
     else:
         sync_redis.publish(wallet, json.dumps({
-            "message": message
-            }))
+            "type": "message",
+            "body": {"message": message}
+        }))
+
+        # log.info("--- api_common.py:publish_message(): published message: " , message)
