@@ -25,7 +25,7 @@ class Rating(BaseModel):
 class BaseProfile(BaseModel):
     wallet: Optional[str]
     uid: Optional[str]
-    name: str
+    name: str = None
     referrer: str = None
     gender: str = None
     register_timestamp: int = 0
@@ -52,12 +52,19 @@ class HairColor(MultiLangEnum):
     black = 'black'
     blond = 'blond'
     red = 'red'
+    ash_blonde = 'ash_blonde'
+    pink = 'pink'
+    purple = 'purple'
+    brunette = 'brunette'
     other = 'other'
 
 class Ethnicity(MultiLangEnum):
     asian = 'asian'
-    australian = 'australian'
+    african = 'african'
+    latin_american = 'latin_american'
     caucasian = 'caucasian'
+    hispanic = 'hispanic'
+    middle_eastern = 'middle_eastern'
 
 class Build(MultiLangEnum):
     athletic = 'athletic'
@@ -65,7 +72,9 @@ class Build(MultiLangEnum):
     fit = 'fit'
     slender = 'slender'
     skinny = 'skinny'
-
+    busty = 'busty'
+    curvy = 'curvy'
+    voluptuous = 'voluptuous'
 
 class Bust(MultiLangEnum):
     a = 'a'
@@ -78,11 +87,12 @@ class Bust(MultiLangEnum):
 
 
 class DressSize(MultiLangEnum):
-    small_minus = 'small_minus'
-    small = 'small'
-    medium = 'medium'
-    large = 'large'
-    large_plus = 'large_plus'
+    six = 'six'
+    eight = 'eight'
+    ten = 'ten'
+    twelve = 'twelve'
+    fourteen = 'fourteen'
+    fourteen_plus = 'fourteen_plus'
 
 class SpeakingLanguage (MultiLangEnum):
     english = 'english'
@@ -90,6 +100,13 @@ class SpeakingLanguage (MultiLangEnum):
     japanese = 'japanese'
     korean = 'korean'
     cantonese = 'cantonese'
+    romanian = 'romanian'
+    fijian = 'fijian'
+    russian = 'russian'
+    french = 'french'
+    italian = 'italian'
+    spanish = 'spanish'
+    arabic = 'arabic'
 
 class PaymentMethod (MultiLangEnum):
     usdt = 'usdt'
@@ -137,11 +154,14 @@ class ProviderProfile(BaseProfile):
     dress_size: Optional[DressSize]
     speaking_language: List[SpeakingLanguage] = []
     payment: List[PaymentMethod] = []
+    instruction_images: Optional[List[str]] = []
+    instruction_text: Optional[str]
 
 
 class ConsumerProfile(BaseProfile):
-    pass
-
+    contact: str = None
+    name: str = None
+    referrer: str = None
 
 class TimeSlot(ProviderProfile):
     slot_id: int  # the slot id, YYYYmmddXX
@@ -240,4 +260,4 @@ class HealthResponse(BaseModel):
     status: str
 
 class NotificationsModel(BaseModel):
-    notifications: List[str] = []
\ No newline at end of file
+    notifications: List[str] = []
