@@ -1,13 +1,13 @@
 import os
 import requests
 
-def get_matched_bdating_wallet_info(consumer_wallet: str):
+def get_matched_bdating_wallet_info(booking_id: str):
     endpoint = os.environ['BDATING_WALLET_API_ENDPOINT']
     endpoint_key = os.environ['BDATING_WALLET_API_KEY']
     return requests.get(
         url=f"{endpoint}match",
         params={
-            'user_wallet': consumer_wallet,
+            'transaction_id': booking_id
         },
         headers={
             "x-api-key": endpoint_key
